﻿using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Serialization;
using Microsoft.CSharp;

namespace Builder
{
    public partial class MainWindow : Form
    {
        public MainWindow ()
        {
            InitializeComponent ();

            createColorPanels (new List<String> { "red", "green", "blue", "yellow", "orange", "lime", "cornflowerblue" });

            createComponent (typeof (Record), typeof (RecordView));
            createComponent (typeof (Field), typeof (FieldView));
            createComponent (typeof (Button));
            createComponent (typeof (Panel));
            createComponent (typeof (TreeView));
            createComponent (typeof (ListView));
            createComponent (typeof (TextBox));
            createComponent (typeof (DataGrid));

            designer.AllowDrop = true;
            treeView.AllowDrop = true;
        }

        private void createColorPanels (List<String> colorNames)
        {
            foreach (var colorName in colorNames)
            {
                Color color = Color.FromName (colorName);
                ColorPanel p = new ColorPanel (color, colorName);
                // int n = colorPage.Controls.Count;
                // p.Left = n * p.Width + (n + 1) * p.Left;
                // colorPage.Controls.Add (p);
                colorToolbar.Items.Add (p);
            }
        }

        private void createComponent (Type type0, Type view0 = null)
        {
            Factory f = new Factory (type0, view0);

            int n = componentPage.Controls.Count;
            f.Left = f.Left + n * (f.Width + f.Left);
            componentPage.Controls.Add (f);
        }

        private void designer_DragEnter (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent (typeof (Color)))
                e.Effect = DragDropEffects.Move;
            // else if (e.Data.GetDataPresent (DataFormats.Text))
            //    e.Effect = DragDropEffects.Copy;
            // else if (e.Data.GetDataPresent ("System.Windows.Forms.Button"))
            //     e.Effect = DragDropEffects.Copy;
            else if (e.Data.GetDataPresent ("Builder.Factory"))
                e.Effect = DragDropEffects.Copy;
            else
                e.Effect = DragDropEffects.None;
        }

        private void designer_DragDrop (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
            {
                Factory factory = e.Data.GetData ("Builder.Factory") as Factory;
                Point location = designer.PointToClient (new Point (e.X, e.Y));
                createObject (factory, null, location);
            }
        }

        private void treeView_DragEnter (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
                e.Effect = DragDropEffects.Copy;
        }

        private void treeView_DragDrop (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
            {
                Factory factory = e.Data.GetData ("Builder.Factory") as Factory;
                Point pt = treeView.PointToClient (new Point (e.X, e.Y));
                TreeNode target = treeView.GetNodeAt (pt);
                createObject (factory, target, new Point (0, 0));
            }
        }

        private void createObject (Factory factory, TreeNode target, Point location)
        {
            Object obj = factory.createInstance ();

            MyNode node = new MyNode ();
            node.Text = obj.ToString ();
            node.Tag = obj;
            node.connect ();

            if (target == null)
                treeView.Nodes.Add (node);
            else
                target.Nodes.Add (node);

            if (obj is Control)
            {
                Control ctl = obj as Control;
                if (location != null)
                {
                    ctl.Left = location.X;
                    ctl.Top = location.Y;
                }

                new Adapter (ctl, this);

                designer.Controls.Add (ctl);
            }

        }

        private void treeView_NodeMouseClick (object sender, TreeNodeMouseClickEventArgs e)
        {
            SelectObject (e.Node.Tag);
        }

        public void SelectObject (object obj)
        {
            if (obj is Common)
                obj = (obj as Common).recallData();

            // obj = new ObjectWrapper (obj);
            propGrid.SelectedObject = obj;
        }

        private void readData (TextWriter stream)
        {

        }

        private void writeData (TextWriter stream)
        {
            foreach (TreeNode node in treeView.Nodes)
            {
                object obj = node.Tag;
                if (obj is Common)
                    obj = (obj as Common).recallData ();

                XmlSerializer writer = new XmlSerializer (obj.GetType ());
                writer.Serialize (stream, obj);
            }
        }

        private void open_Click (object sender, EventArgs e)
        {
            if (openDialog.ShowDialog () == DialogResult.OK)
            {
                StreamReader stream = new StreamReader (openDialog.FileName);
                // XmlSerializer reader = new XmlSerializer (typeof (MyObject));
                // obj = (MyObject)reader.Deserialize (stream);
                stream.Close ();

                // AddObject (obj);
            }
        }

        private void save_Click (object sender, EventArgs e)
        {
            if (saveDialog.ShowDialog () == DialogResult.OK)
            {
                StreamWriter stream = new StreamWriter (saveDialog.FileName);
                // XmlSerializer writer = new XmlSerializer (typeof (MyObject));
                // writer.Serialize (stream, obj);
                writeData (stream);
                stream.Close ();
            }
        } 

        private void viewXml_Click (object sender, EventArgs e)
        {
            StringWriter stream = new StringWriter ();
            writeData (stream);
            info.Text = stream.ToString ();
        }

        private void viewData_Click (object sender, EventArgs e)
        {
            new DataWindow ().Show ();
        }

        private void quit_Click(object sender , EventArgs e)
        {
            Close ();
        }
        
        /* Compilation */
        
        void displayAssembly (Assembly a)
        {
            codeTree.Nodes.Clear ();
            foreach (Module m in a.GetModules ())
               displayModule (codeTree.Nodes, m);
        }
        
        void displayModule (TreeNodeCollection target, Module m)
        {
            TreeNode node = new TreeNode();
            node.Text = "module " + m.Name;
            node.Tag = m;
            foreach (Type t in m.GetTypes ())
                displayType (node, t);
            target.Add (node);
        }

        void displayType (TreeNode target, Type t)
        {
            TreeNode node = new TreeNode();
            node.Text = "type " + t.Name;
            node.Tag = t;
            foreach (MemberInfo m in t.GetMembers ())
                displayMember (node, m);
            target.Nodes.Add (node);
        }
        
        void displayMember (TreeNode target, MemberInfo m)
        {
            TreeNode node = new TreeNode();
            node.Text = m.Name;
            node.Tag = m;
            if (m is MethodBase)
            {
                MethodBase b = m as MethodBase;
                foreach (ParameterInfo p in b.GetParameters ())
                    displayParameter (node, p);
            }
            if (m is MethodInfo)
            {
                displayResult (node, m as MethodInfo);
            }
            target.Nodes.Add (node);
        }
        
        void displayParameter (TreeNode target, ParameterInfo p)
        {
            TreeNode node = new TreeNode();
            node.Text = p.Name + " : " + p.ParameterType.ToString ();
            node.Tag = p;
            target.Nodes.Add (node);
        }
        
        void displayResult (TreeNode target, MethodInfo m)
        {
            TreeNode node = new TreeNode();
            node.Text = "result " + m.ReturnType.ToString ();
            target.Nodes.Add (node);
        }
        
        void codeTree_NodeMouseClick (object sender, TreeNodeMouseClickEventArgs e)
        {
            propGrid.SelectedObject = e.Node.Tag;
        }
        
        void compile_Click (object sender, EventArgs e)
        {
            info.Clear ();
            centralTabs.SelectedTab = editorTab;
            string[] code = { editor.Text };
            
            string[] code0 = {
                                "using System; using System.Data;",
                                "namespace CodeFromFile"+
                                "{"+
                                "   public class CodeFromFile"+
                                "   {"+
                                "       static public int Add(int a ,int b)"+
                                "       {"+
                                "           return a+b;"+
                                "       }"+
                                "   }"+
                                "}"
                         };

               
            CompilerParameters CompilerParams = new CompilerParameters ();

            CompilerParams.GenerateInMemory = true;
            CompilerParams.TreatWarningsAsErrors = false;
            CompilerParams.GenerateExecutable = false;
            CompilerParams.CompilerOptions = "/optimize";
      
            string[] references = { "System.dll", "System.Data.dll" };
            CompilerParams.ReferencedAssemblies.AddRange (references);
      
            CSharpCodeProvider provider = new CSharpCodeProvider ();
            CompilerResults compile = provider.CompileAssemblyFromSource (CompilerParams, code);
      
            if (compile.Errors.HasErrors)
            {
                string text = "Compile error: ";
                foreach (CompilerError ce in compile.Errors)
                {
                    info.AppendText (ce.ToString () + "\r\n");
                    text += "\r\n" + ce.ToString ();
                }
                // throw new Exception (text);
            }
            else
            {
                leftTabs.SelectedTab = codeTab;
                displayAssembly (compile.CompiledAssembly);
                
                Module module = compile.CompiledAssembly.GetModules ()[0];
                Type mt = null;
                MethodInfo meth = null;
      
                if (module != null)
                    mt = module.GetType ("CodeFromFile.CodeFromFile");
      
                if (mt != null)
                    meth = mt.GetMethod ("Add");
            
                if (meth != null)
                {
                    int result = (int)meth.Invoke (null, new object[] { 5, 10 });
                    info.AppendText ("result = " + result);
                }
            }
       
        }
        
    }
}
